@@ -1,8 +1,10 @@
 from __future__ import annotations
 
 import copy
+import math
 import re
 from collections import Counter, OrderedDict
+from types import SimpleNamespace
 from typing import Callable, Optional, Union
 
 import elementy
@@ -86,12 +88,12 @@ class Alloy:
         if structure is None and composition_structure is not None:
             structure = composition_structure
 
+        self.constraints = None
+
         self._structure = None
         if structure is not None:
             self.structure = structure
 
-        self.constraints = None
-
         if rescale:
             self.rescale()
 
@@ -156,50 +158,102 @@ class Alloy:
     def structure(self, structure):
 
         if isinstance(structure, str):
-            structure = copy.deepcopy(get_prototype(structure))
+            try:
+                structure = copy.deepcopy(get_prototype(structure))
+            except:
+                self._structure = SimpleNamespace(**{"name": structure})
+                return
+
         self._structure = structure
 
+        structure_elements_ordered = sorted(
+            self._structure.composition,
+            key=lambda x: self._structure.composition[x],
+            reverse=True,
+        )
+
+        for i in range(len(self.elements)):
+            if i not in structure_elements_ordered:
+                structure_elements_ordered.append(i)
+
         if not all(
             value == list(self.original_composition.values())[0]
             for value in self.original_composition.values()
         ):
-            structure_elements_ordered = sorted(
-                self.structure.composition,
-                key=lambda x: self.structure.composition[x],
-                reverse=True,
-            )
+            digits = 0
+            for element in self.composition:
+                digits = max([digits, len(str(self.composition[element])) - 2])
 
-            composition_elements_ordered = [
-                list(self.original_composition.keys()).index(e)
-                for e in self.original_composition
-            ]
-
-            index_translation = {
-                i: min([j, max(composition_elements_ordered)])
-                for i, j in zip(
-                    composition_elements_ordered,
-                    structure_elements_ordered,
-                )
-            }
-
-            for b in self.structure.basis:
-                if b["element"] in index_translation:
-                    b["element"] = index_translation[b["element"]]
+        self.structure_element_translation = {}
+        for i, element in enumerate(self.elements):
+            if i >= len(self.structure.elements):
+                structure_element_index = i % len(self.structure.elements)
+            else:
+                structure_element_index = i
+            self.structure_element_translation[i] = structure_element_index
 
         structure_composition = {}
-        for structure_element_index in self.structure.elements:
-            if structure_element_index >= len(self.elements):
-                element_index = len(self.elements) - 1
-            else:
-                element_index = structure_element_index
+        for i in self.structure_element_translation:
+            if i not in structure_composition:
+                structure_composition[i] = 0
+            structure_composition[i] += self.structure.composition[
+                self.structure_element_translation[i]
+            ]
 
-            if self.elements[element_index] not in structure_composition:
-                structure_composition[self.elements[element_index]] = 0
-            structure_composition[
-                self.elements[element_index]
-            ] += self.structure.composition[element_index]
+        if not all(
+            value == list(self.original_composition.values())[0]
+            for value in self.original_composition.values()
+        ):
+            total = sum(structure_composition.values())
+            for i in structure_composition:
+                structure_composition[i] /= total
+                structure_composition[i] = multiple_round(
+                    structure_composition[i], 10 ** (-digits - 1), digits + 1
+                )
 
-        self.composition = structure_composition
+            self.structure_element_translation = {}
+            taken = []
+
+            for element in self.composition:
+                for i in structure_composition:
+                    if i not in taken and math.isclose(
+                        self.composition[element],
+                        structure_composition[i],
+                        abs_tol=10 ** (-digits),
+                    ):
+                        self.structure_element_translation[element] = i
+                        taken.append(i)
+                        break
+
+            translated_structure_composition = {}
+            for element in self.structure_element_translation:
+                translated_structure_composition[
+                    element
+                ] = structure_composition[
+                    self.structure_element_translation[element]
+                ]
+
+            if self.constraints is None:
+                self.constraints = {
+                    "digits": digits,
+                    "percentage_step": (10**-digits),
+                }
+        else:
+            translated_structure_composition = {}
+            for i in self.structure_element_translation:
+                translated_structure_composition[
+                    self.elements[i]
+                ] = structure_composition[i]
+
+            for i in range(len(self.elements)):
+                self.structure_element_translation[
+                    self.elements[i]
+                ] = self.structure_element_translation[i]
+
+                del self.structure_element_translation[i]
+
+        self.composition = translated_structure_composition
+        self.rescale()
 
     @property
     def elements(self) -> list:
@@ -216,15 +270,16 @@ class Alloy:
         :group: alloy
         """
         if self.structure is not None:
-            unique_elements = []
-            for b in self.structure.original_basis:
-                if b["element"] < len(self.original_elements) and (
-                    self.original_elements[b["element"]] not in unique_elements
-                ):
-                    unique_elements.append(
-                        self.original_elements[b["element"]]
-                    )
-            return unique_elements
+            sorted_translation = sorted(
+                self.structure_element_translation.items(),
+                key=lambda x: x[1],
+            )
+
+            elements = []
+            for pair in sorted_translation:
+                elements.append(pair[0])
+            return elements
+
         else:
             return self.elements
 
@@ -286,12 +341,16 @@ class Alloy:
 
         :group: alloy.utils
         """
-        for element in self.constraints["local_percentages"]:
-            if element not in self.composition:
-                if self.constraints["local_percentages"][element]["min"] > 0:
-                    self.composition[element] = self.constraints[
-                        "local_percentages"
-                    ][element]["min"]
+        if "local_percentages" in self.constraints:
+            for element in self.constraints["local_percentages"]:
+                if element not in self.composition:
+                    if (
+                        self.constraints["local_percentages"][element]["min"]
+                        > 0
+                    ):
+                        self.composition[element] = self.constraints[
+                            "local_percentages"
+                        ][element]["min"]
 
     def rescale(self):
         """Adjust elemental percentages to match constraints
@@ -399,17 +458,17 @@ class Alloy:
             self.clamp_composition()
             self.round_composition()
 
+        self.reorder_composition()
+
         if not constraints_applied:
             self.clamp_composition()
         self.round_composition()
 
-        self.reorder_composition()
-
         delattr(self, "rescaling")
 
     def reorder_composition(self):
         ordered_elements = self.elements
-        if self.constraints is not None:
+        if self.constraints is not None and "percentages" in self.constraints:
             ordered_elements = sorted(
                 self.composition,
                 key=lambda x: (
@@ -481,7 +540,10 @@ class Alloy:
         """
 
         satisfied = True
-        if self.constraints is not None:
+        if (
+            self.constraints is not None
+            and "local_percentages" in self.constraints
+        ):
 
             discrepancy = np.abs(
                 1
@@ -498,7 +560,8 @@ class Alloy:
                 # print()
                 satisfied = False
 
-            if satisfied:
+            if satisfied and "local_percentages" in self.constraints:
+
                 for element in self.constraints["local_percentages"]:
                     if element in self.composition:
                         if (
@@ -584,12 +647,20 @@ class Alloy:
                         break
 
             if satisfied:
-                if len(self.composition) > self.constraints["max_elements"]:
+                if (
+                    "max_elements" in self.constraints
+                    and len(self.composition)
+                    > self.constraints["max_elements"]
+                ):
                     satisfied = False
                     # print("too many elements")
                     # print(self.composition)
                     # print()
-                elif len(self.composition) < self.constraints["min_elements"]:
+                elif (
+                    "min_elements" in self.constraints
+                    and len(self.composition)
+                    < self.constraints["min_elements"]
+                ):
                     satisfied = False
                     # print("too few elements")
                     # print(self.composition)
@@ -631,7 +702,10 @@ class Alloy:
                     clamped_value, percentage_step, digits
                 )
 
-                if self.constraints is not None:
+                if (
+                    self.constraints is not None
+                    and "percentages" in self.constraints
+                ):
                     if element in self.constraints["percentages"]:
                         clamped_value = max(
                             clamped_value,
@@ -654,7 +728,10 @@ class Alloy:
                     if element not in self.composition:
                         continue
 
-                    if self.constraints is not None:
+                    if (
+                        self.constraints is not None
+                        and "percentages" in self.constraints
+                    ):
                         if element in self.constraints["percentages"]:
                             change = max(
                                 [
@@ -700,7 +777,7 @@ class Alloy:
         :group: alloy.utils
         """
 
-        if self.constraints is None:
+        if self.constraints is None or "percentages" not in self.constraints:
             return
 
         tmp_percentages = copy.deepcopy(self.constraints["percentages"])
@@ -866,7 +943,7 @@ class Alloy:
         """
 
         composition_str = ""
-        for element in self.elements:
+        for element in self.composition:
             percentage_str = str(self.composition[element] * 100.0)
 
             split_str = percentage_str.split(".")
@@ -877,6 +954,7 @@ class Alloy:
                 decimal_places = len(
                     str(self.composition[element]).split(".")[1]
                 )
+
                 percentage_str = str(
                     round(float(percentage_str), decimal_places)
                 )
@@ -979,7 +1057,10 @@ class Alloy:
 
                 precedence = {}
                 for element in self.elements:
-                    if element in self.constraints["local_percentages"]:
+                    if (
+                        "local_percentages" in self.constraints
+                        and element in self.constraints["local_percentages"]
+                    ):
                         precedence[element] = self.constraints[
                             "local_percentages"
                         ][element]["precedence"]
@@ -1005,7 +1086,8 @@ class Alloy:
                 constraints_violated = False
                 if self.constraints is not None:
                     if (
-                        filtered_precedence_order[i]
+                        "local_percentages" in self.constraints
+                        and filtered_precedence_order[i]
                         in self.constraints["local_percentages"]
                     ):
                         if decimal_part_index < len(integer_parts):
