@@ -12,16 +12,20 @@ from bokeh.models import (BasicTicker, ColorBar, ColumnDataSource,
 from bokeh.plotting import figure
 from bokeh.sampledata.periodic_table import elements
 from bokeh.transform import dodge
+from matplotlib.figure import Figure
 
 import metallurgy as mg
 
 
 def plot(
-    alloys: list,
-    data: list,
+    alloys: Optional = None,
+    data: Optional[list] = [],
+    elements: Optional[List[str]] = None,
+    property_name: str = None,
     label: str = None,
     labels: Optional[List[str]] = None,
     save_path: str = None,
+    step: Optional = None,
 ):
     """An automatic interface to metallurgy's alloy system plotting
     functions.
@@ -38,6 +42,22 @@ def plot(
 
     """
 
+    if (alloys is None and elements is None) or (
+        alloys is not None and elements is not None
+    ):
+        raise ValueError("Must provide either alloys or elements.")
+
+    if elements is not None:
+        if len(elements) < 4:
+            alloys, percentages = mg.generate.system(elements, step=step)
+        else:
+            alloys = mg.generate.system(elements, step=step)
+
+    if len(data) == 0 and property_name is None:
+        raise ValueError("Must provide either data or a property_name.")
+    elif property_name is not None:
+        data = mg.calculate(alloys, property_name)
+
     if isinstance(alloys[0], mg.Alloy):
         num_elements = len(mg.analyse.find_unique_elements(alloys))
     elif isinstance(alloys[0], list) and isinstance(alloys[0][0], mg.Alloy):
@@ -86,7 +106,7 @@ def binary(
         The alloys across the binary composition
     """
 
-    fig = plt.figure()
+    fig = Figure()
     ax1 = fig.add_subplot(111)
 
     elements = []
@@ -175,17 +195,15 @@ def binary(
     if ylabel is not None:
         ax1.set_ylabel(ylabel)
 
-    plt.tight_layout()
+    fig.tight_layout()
 
-    plt.gcf().set_dpi(300)
+    # fig.gcf().set_dpi(300)
     if save_path is not None:
-        plt.savefig(save_path)
+        fig.savefig(save_path)
     else:
-        plt.show()
+        fig.show()
 
-    plt.cla()
-    plt.clf()
-    plt.close()
+    fig.clf()
 
 
 def ternary(
@@ -296,13 +314,13 @@ def ternary(
 
     tax.get_axes().set_aspect(1)
     tax._redraw_labels()
-    plt.gcf().set_dpi(300)
+    # plt.gcf().set_dpi(300)
 
     if ax is None:
         if save_path is None:
             tax.show()
         else:
-            tax.savefig(save_path + ".png")
+            tax.savefig(save_path)
 
         tax.close()
         figure.clf()
@@ -335,7 +353,7 @@ def quaternary(
     numGridCells = columns * rows
     gridExcess = numGridCells - len(quaternary_alloys)
 
-    fig = plt.figure(figsize=(4 * columns, 4 * rows))
+    fig = Figure(figsize=(4 * columns, 4 * rows))
 
     lastAx = None
     for i in reversed(range(len(quaternary_alloys))):
